@@ -35,6 +35,7 @@
 #include "gupnp-context-private.h"
 #include "gupnp-error.h"
 #include "gupnp-error-private.h"
+#include "gupnp-xml-doc.h"
 #include "http-headers.h"
 #include "xml-util.h"
 
@@ -51,7 +52,7 @@ struct _GUPnPServiceInfoPrivate {
 
         SoupURI *url_base;
 
-        XmlDocWrapper *doc;
+        GUPnPXMLDoc *doc;
 
         xmlNode *element;
 
@@ -124,10 +125,7 @@ gupnp_service_info_set_property (GObject      *object,
 
                 break;
         case PROP_DOCUMENT:
-                info->priv->doc = g_value_get_object (value);
-                if (info->priv->doc)
-                        g_object_ref_sink (info->priv->doc);
-
+                info->priv->doc = g_value_dup_object (value);
                 break;
         case PROP_ELEMENT:
                 info->priv->element = g_value_get_pointer (value);
@@ -186,7 +184,7 @@ gupnp_service_info_dispose (GObject *object)
         if (info->priv->context) {
                 SoupSession *session;
 
-                session = _gupnp_context_get_session (info->priv->context);
+                session = gupnp_context_get_session (info->priv->context);
 
                 while (info->priv->pending_gets) {
                         GetSCPDURLData *data;
@@ -213,6 +211,8 @@ gupnp_service_info_dispose (GObject *object)
                 g_object_unref (info->priv->doc);
                 info->priv->doc = NULL;
         }
+
+        G_OBJECT_CLASS (gupnp_service_info_parent_class)->dispose (object);
 }
 
 static void
@@ -227,6 +227,8 @@ gupnp_service_info_finalize (GObject *object)
         g_free (info->priv->service_type);
 
         soup_uri_free (info->priv->url_base);
+
+        G_OBJECT_CLASS (gupnp_service_info_parent_class)->finalize (object);
 }
 
 static void
@@ -347,7 +349,7 @@ gupnp_service_info_class_init (GUPnPServiceInfoClass *klass)
                                       "Document",
                                       "The XML document related to this "
                                       "service",
-                                      TYPE_XML_DOC_WRAPPER,
+                                      GUPNP_TYPE_XML_DOC,
                                       G_PARAM_WRITABLE |
                                       G_PARAM_CONSTRUCT_ONLY |
                                       G_PARAM_STATIC_NAME |
@@ -580,19 +582,19 @@ gupnp_service_info_get_introspection (GUPnPServiceInfo *info,
         http_request_set_user_agent (msg);
 
         /* Send off the message */
-        session = _gupnp_context_get_session (info->priv->context);
+        session = gupnp_context_get_session (info->priv->context);
 
         status = soup_session_send_message (session, msg);
         if (!SOUP_STATUS_IS_SUCCESSFUL (status)) {
-                set_server_error (error, msg);
+                _gupnp_error_set_server_error (error, msg);
 
                 g_object_unref (msg);
 
                 return NULL;
         }
 
-        scpd = xmlParseMemory (msg->response_body->data,
-                               msg->response_body->length);
+        scpd = xmlRecoverMemory (msg->response_body->data,
+                                 msg->response_body->length);
 
         g_object_unref (msg);
 
@@ -632,8 +634,8 @@ got_scpd_url (SoupSession    *session,
         if (SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
                 xmlDoc *scpd;
 
-                scpd = xmlParseMemory (msg->response_body->data,
-                                       msg->response_body->length);
+                scpd = xmlRecoverMemory (msg->response_body->data,
+                                         msg->response_body->length);
                 if (scpd) {
                         introspection = gupnp_service_introspection_new (scpd);
 
@@ -647,7 +649,7 @@ got_scpd_url (SoupSession    *session,
                                          "Could not parse SCPD");
                 }
         } else
-                error = new_server_error (msg);
+                error = _gupnp_error_new_server_error (msg);
 
         data->info->priv->pending_gets =
                 g_list_remove (data->info->priv->pending_gets, data);
@@ -725,7 +727,7 @@ gupnp_service_info_get_introspection_async
                 g_list_prepend (info->priv->pending_gets,
                                 data);
 
-        session = _gupnp_context_get_session (info->priv->context);
+        session = gupnp_context_get_session (info->priv->context);
 
         soup_session_queue_message (session,
                                     data->message,
